/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*exported popupController*/

var popupController = (function() {
    // Predefined Values
    var MAX_Z_INDEX = 1000,

    // Private Variables
        self = {},
        divPopup = null,
        divDetail = null,
        divFooter = null,
        popupStatus = "Deactive";

    // Public Variables

    // Private Functions
    /**
     * Removes all child of the element.
     * @private
     * @param {Object} elm - The object wants to be emptied
     * @return {Object} The emptied element
     */
    function emptyElement(elm) {
        while (elm.firstChild) {
            elm.removeChild(elm.firstChild);
        }

        return elm;
    }

    /**
     * Checks whether the object is child of another object or not.
     * @private
     * @param {Object} c - The child object to be checked
     * @param {Object} p - The assumed parent object
     * @return {boolean} true if c is the child of p
     */
    function childOf(c, p) {
        while (c && c !== p) {
            c = c.parentNode;
        }
        return !!c;
    }
    
    /**
     * Sets the predefined default styles of target object.
     * @private
     * @param {Object} elm - The target object to be set styles
     * @param {string} type - The type of predefined style
     */
    function setObjectDefaultStyle(elm, type) {
        switch (type) {
        case "Popup":
            elm.style.position = "absolute";
            elm.style.top = "0px";
            elm.style.left = "0px";
            elm.style.width = "100%";
            elm.style.height = "100%";
            elm.style.backgroundColor = "#000000";
            elm.style.visibility = "hidden";
            elm.style.zIndex = MAX_Z_INDEX;
            break;
        case "DetailBox":
            elm.style.textAlign = "center";
            elm.style.display = "table";
            elm.style.width = "100%";
            elm.style.height = "100%";
            break;
        case "Detail":
            elm.style.verticalAlign = "middle";
            elm.style.color = "#ffffff";
            elm.style.display = "table-cell";
            break;
        case "Footer":
            elm.style.width = "100%";
            elm.style.height = "50px";
            elm.style.marginTop = "-50px";
            elm.style.display = "table";
            elm.style.verticalAlign = "middle";
            break;
        case "ButtonFull":
            elm.style.width = "100%";
            elm.style.backgroundColor = "rgba(39, 119, 217, 1)";
            elm.style.display = "table-cell";
            elm.style.textAlign = "center";
            elm.style.verticalAlign = "middle";
            elm.style.color = "#ffffff";
            break;
        case "ButtonHalf":
            elm.style.width = "50%";
            elm.style.backgroundColor = "rgba(39, 119, 217, 1)";
            elm.style.display = "table-cell";
            elm.style.textAlign = "center";
            elm.style.verticalAlign = "middle";
            elm.style.color = "#ffffff";
            break;
        default:
            break;
        }
    }

    // Public functions
    /**
     * Creates a popup with an id of the existing element, or newly created element.
     * @public
     */
    self.createPopup = function (popup, detail, footer) {
        var divBox;

        if (popup && typeof popup === "string") {
            divPopup = document.querySelector("#" + popup);
        } else {
            divPopup = document.createElement("div");
            setObjectDefaultStyle(divPopup, "Popup");
            document.body.appendChild(divPopup);
        }
        divPopup.style.visibility = "hidden";

        if (detail && typeof detail === "string") {
            divDetail = document.querySelector("#" + detail);
            if (childOf(divDetail, divPopup) === false) {
                console.log("Detail(" + detail + ") is not child of Popup(" + popup + ")");
            }
        } else {
            divBox = document.createElement("div");
            divDetail = document.createElement("div");
            setObjectDefaultStyle(divBox, "DetailBox");
            setObjectDefaultStyle(divDetail, "Detail");
            divBox.appendChild(divDetail);
            divPopup.appendChild(divBox);
        }

        if (footer && typeof footer === "string") {
            divFooter = document.querySelector("#" + footer);
            if (childOf(divFooter, divPopup) === false) {
                console.log("Detail(" + detail + ") is not child of Popup(" + popup + ")");
            }
        } else {
            divFooter = document.createElement("div");
            setObjectDefaultStyle(divFooter, "Footer");
            divPopup.appendChild(divFooter);
        }
    };

    /**
     * Checks whether the popupStatus state is active state.
     * @public
     * @return {boolean} true if PopupControl is in active state
     */
    self.isActive = function() {
        if (popupStatus === "Active") {
            return true;
        }

        return false;
    };

    /**
     * Closes the popup.
     * @public
     * @return {boolean} true if the popup was successfully closed
     */
    self.closePopup = function() {
        if (popupStatus === "Deactive") {
            return false;
        }

        divPopup.style.visibility = "hidden";
        popupStatus = "Deactive";

        return true;
    };

    /**
     * Sets the position of popup.
     * @public
     * @param {string} pos - A position of the popup to be placed.
     * @return {boolean} true if PopupControl is placed to the position.
     */
    self.setPosition = function(pos) {
        var screenWidth,
            screenHeight,
            divWidth,
            divHeight;

        if (pos) {
            switch (pos) {
            case "Center":
                screenWidth = document.body.clientWidth;
                screenHeight = document.body.clientHeight;
                divWidth = divPopup.offsetWidth;
                divHeight = divPopup.offsetHeight;
                divPopup.style.left = ((screenWidth - divWidth) / 2) + "px";
                divPopup.style.top = ((screenHeight - divHeight) / 2) + "px";
                break;
            case "BottomCenter":
                screenWidth = document.body.clientWidth;
                screenHeight = document.body.clientHeight;
                divWidth = divPopup.offsetWidth;
                divHeight = divPopup.offsetHeight;
                divPopup.style.left = ((screenWidth - divWidth) / 2) + "px";
                divPopup.style.top = (screenHeight - divHeight) + "px";
                break;
            default:
                break;
            }

            return true;
        }

        return false;
    };

    /**
     * Shows the popup.
     * @public
     * @param {string} message - Message to be shown in popup detail
     * @param {string} type - Type of the popup.
     * @param {Object} [callbackList] - List of callbacks to be linked with buttons in popup footer.
     * @param {Function} [callbackList.ok] - callback function for OK button.
     * @param {Function} [callbackList.cancel] - callback function for Cancel button.
     * @return {boolean} true if the popup was successfully showed
     */
    self.showPopup = function(message, type, callbackList) {
        // Show Popup
        var objButton;

        if (!message || !type || self.isActive()) {
            return false;
        }

        switch (type) {
        case "OK":
            emptyElement(divFooter);
            objButton = document.createElement("div");
            setObjectDefaultStyle(objButton, "ButtonFull");
            objButton.appendChild(document.createTextNode("OK"));
            if (callbackList && callbackList.cbOK && typeof(callbackList.cbOK) === "function") {
                objButton.addEventListener("click", function() {
                    callbackList.ok();
                    self.closePopup();
                });
            } else {
                objButton.addEventListener("click", function() {
                    self.closePopup();
                });
            }
            divFooter.appendChild(objButton);

            emptyElement(divDetail);
            divDetail.appendChild(document.createTextNode(message));

            divPopup.style.visibility = "visible";
            popupStatus = "Active";

            break;
        case "OKCancel":
            emptyElement(divFooter);
            objButton = document.createElement("div");
            setObjectDefaultStyle(objButton, "ButtonHalf");
            objButton.appendChild(document.createTextNode("OK"));
            // register OK Button
            if (callbackList && callbackList.cbOK && typeof(callbackList.cbOK) === "function") {
                objButton.addEventListener("click", function() {
                    callbackList.ok();
                    self.closePopup();
                });
            } else {
                objButton.addEventListener("click", function() {
                    self.closePopup();
                });
            }
            divFooter.appendChild(objButton);
            objButton = document.createElement("div");
            setObjectDefaultStyle(objButton, "ButtonHalf");
            objButton.appendChild(document.createTextNode("Cancel"));
            // register cancel button
            if (callbackList && callbackList.cbCancel && typeof(callbackList.cbCancel) === "function") {
                objButton.addEventListener("click", function() {
                    callbackList.cancel();
                    self.closePopup();
                });
            } else {
                objButton.addEventListener("click", function() {
                    self.closePopup();
                });
            }
            divFooter.appendChild(objButton);

            emptyElement(divDetail);
            divDetail.appendChild(document.createTextNode(message));

            divPopup.style.visibility = "visible";
            popupStatus = "Active";

            break;
        default:
            emptyElement(divFooter);
            objButton = document.createElement("div");
            setObjectDefaultStyle(objButton, "ButtonFull");
            objButton.appendChild(document.createTextNode("Close"));
            objButton.addEventListener("click", function() {
                self.closePopup();
            });
            divFooter.appendChild(objButton);

            emptyElement(divDetail);
            divDetail.appendChild(document.createTextNode(message));

            divPopup.style.visible = "visible";
            popupStatus = "Active";
        }

        return true;
    };

    return self;
}());